from django.contrib.auth.models import AbstractUser
from django.db import models
import re
from django.core.exceptions import ValidationError
from datetime import date, timedelta

def validate_username(value):
    if not re.match(r'^[a-zA-Z0-9._-]+$', value):
        raise ValidationError("Username hanya boleh berisi huruf, angka, '.', '_', atau '-'.")
    
def validate_password(value):
    if len(value) < 8 or not re.search(r'[\W_]', value):
        raise ValidationError("Password harus minimal 8 karakter dan mengandung karakter spesial.")

def validate_birthdate(value):
    min_birthdate = date.today() - timedelta(days=12*365)
    if value > min_birthdate:
        raise ValidationError("Usia minimal adalah 12 tahun.")

def validate_phone(value):
    if not re.match(r'^62[0-9]{6,13}$', value):
        raise ValidationError("Nomor HP harus dalam format 62xxxxxxxx dan panjangnya 8-15 angka.")

def validate_nomor_rangka(value):
    if not re.match(r'^[A-Za-z]{5}[A-Za-z0-9]{10}$', value):
        raise ValidationError("Nomor rangka harus diawali 5 huruf, lalu 10 karakter alfanumerik.")

# MODEL USER
class User(AbstractUser):
    username = models.CharField(max_length=255, unique=True, validators=[validate_username])
    password = models.CharField(max_length=255, validators=[validate_password])
    birthdate = models.DateField(validators=[validate_birthdate])
    phone_number = models.CharField(max_length=15, validators=[validate_phone])
    email = models.EmailField(unique=True)
    blog_url = models.URLField()
    bio = models.TextField(max_length=1000, blank=True)

    def __str__(self):
        return self.username

# MODEL TRANSPORTASI
class Transportasi(models.Model):
    LOKASI_DUDUK_CHOICES = [
        ('Jendela', 'Jendela'),
        ('Tengah', 'Tengah'),
        ('Gang', 'Gang'),
    ]

    lokasi_duduk = models.CharField(
        max_length=10, 
        choices=LOKASI_DUDUK_CHOICES,
        verbose_name="Lokasi Duduk"
    )

    nomor_rangka = models.CharField(
        max_length=15, 
        unique=True, 
        validators=[validate_nomor_rangka],  
        verbose_name="Nomor Rangka Kendaraan"
    )

    def __str__(self):
        return f"{self.nomor_rangka} - {self.lokasi_duduk}"
